@@ -28,7 +28,6 @@ typedef enum {
     BDJ_EVENT_PLAYITEM,
     BDJ_EVENT_ANGLE,
     BDJ_EVENT_SUBTITLE,
-    BDJ_EVENT_PIP,
     BDJ_EVENT_END_OF_PLAYLIST,
     BDJ_EVENT_PTS,
     BDJ_EVENT_VK_KEY,
@@ -40,6 +39,9 @@ typedef enum {
     BDJ_EVENT_STOP,
 
     BDJ_EVENT_RATE,
+    BDJ_EVENT_AUDIO_STREAM,
+    BDJ_EVENT_SECONDARY_STREAM,
+    BDJ_EVENT_UO_MASKED,
 } BDJ_EVENT;
 
 typedef struct {
